from argparse import ArgumentParser, SUPPRESS

from carrot_mc.carrot import CarrotService


def main():
    commands = [
        StatusCommand(),
        #ListCommand(),
        InstallCommand(),
        InitCommand(),
        UpdateCommand()
    ]

    ap = ArgumentParser(prog='carrot')

    subparsers = ap.add_subparsers()

    for cmd in commands:
        cmd.register_help(subparsers)

    args = ap.parse_args()

    if hasattr(args, 'func'):
        args.func(args)
    else:
        ap.print_help()


class Command(object):
    carrot_service = CarrotService()

    def register_help(self, subparsers):
        raise NotImplementedError()

    def handle_args(self, args):
        raise NotImplementedError()


class StatusCommand(Command):
    def register_help(self, subparsers):
        parser = subparsers.add_parser(
            'status',
            help='Display a summary of current mod directory'
        )

        parser.set_defaults(func=self.handle_args)

    def handle_args(self, args):
        if self.carrot_service.initialized():
            print('Mod repo status: OK')
            self.carrot_service.status(args)
        else:
            print('Mod repo status: INVALID')
            print('This directory does not appear to be a valid mod repo.')


class ListCommand(Command):
    def register_help(self, subparsers):
        parser = subparsers.add_parser(
            'list',
            help='List mods installed in current directory'
        )

        parser.set_defaults(func=self.handle_args)

        # TODO: Implement


class InitCommand(Command):
    def register_help(self, subparsers):
        parser = subparsers.add_parser(
            'init',
            help='Initialize a mod repo in current directory.'
        )

        parser.add_argument(
            '--name',
            help='Name of this repo/modpack'
        )

        parser.add_argument(
            '--mc_version',
            help='Minecraft version',
            required=True
        )

        parser.add_argument(
            '--channel',
            help='Mod release channel',
            choices=['Alpha', 'Beta', 'Release'],
            default='Beta'
        )

        parser.set_defaults(func=self.handle_args)

    def handle_args(self, args):
        if self.carrot_service.initialized():
            print('ERROR! This directory is already a mod repo.')
        else:
            self.carrot_service.initialize(vars(args))
            print('Repo initialized.')


class InstallCommand(Command):
    def register_help(self, subparsers):
        parser = subparsers.add_parser(
            'install',
            help='Install a mod and its dependencies'
        )

        parser.add_argument(
            'mod_key',
            help='"Key" id of a mod *exactly* as used in e.g. CurseForge URLs. '
                 'If no mod matches exactly, a list of possible matches will be displayed.',
            nargs='+'
        )

        parser.add_argument(
            '--channel',
            help='Specify channel (Alpha/Beta/Release) to use when installing. '
                 'Overrides settings from mods.json.',
            choices=['Alpha', 'Beta', 'Release'],
            default=None
        )

        parser.add_argument(
            '--upgrade',
            help='Specify that upgrading mods to newer versions is allowed. '
                 'By default when a mod or dependency is already installed and a new version is found '
                 'during installation, it will be skipped for safety reasons.',
            nargs='?',
            const=True
        )

        parser.add_argument(
            '--downgrade',
            help='Specify that a downgrade to an older version is allowed. '
                 'This can happen e.g. if you explicitly request a Release channel but '
                 'already have a newer Beta file installed.',
            nargs='?',
            const=True
        )

        parser.set_defaults(func=self.handle_args)
    
    def handle_args(self, args):
        self.carrot_service.install(args)


class UpdateCommand(Command):
    def register_help(self, subparsers):
        parser = subparsers.add_parser(
            'update',
            help='Update a currently-installed mod (and its dependencies) or all mods to newer/older version and/or different channel.'
        )

        parser.add_argument(
            'mod_key',
            help='"Key" id of a mod to update (along with its dependencies).',
            nargs='?',
            default=None
        )

        parser.add_argument(
            '--channel',
            help='Specify target channel (Alpha/Beta/Release) to use when updating. '
                 'Overrides settings from mods.json.',
            choices=['Alpha', 'Beta', 'Release'],
            default=None
        )

        # Hidden "option" for compatibility in the installer logic
        parser.add_argument(
            '--upgrade',
            help=SUPPRESS,
            nargs='?',
            default=True
        )

        parser.add_argument(
            '--downgrade',
            help='Specify that downgrades to an older version are allowed. '
                 'This can happen e.g. if you explicitly request a Release channel but '
                 'already have newer versions of mods from the Beta channel installed.',
            nargs='?',
            const=True
        )

        parser.set_defaults(func=self.handle_args)

    def handle_args(self, args):
        self.carrot_service.update(args)


if __name__ == '__main__':
    main()


from carrot_mc.data import Automappable, Autoproperty
from carrot_mc.meta import VERSION


class CarrotModel(Automappable):
    def __init__(self):
        self.version = Autoproperty(int, 1)
        self.carrot_version = Autoproperty(str, VERSION)
        self.name = Autoproperty(str, '')
        self.mc_version = Autoproperty(str)
        self.channel = Autoproperty(str, 'Beta')
        self.mods = Autoproperty(InstalledModModel, [], True)


class BaseModModel(Automappable):
    def __init__(self):
        self.key = Autoproperty(str)
        self.id = Autoproperty(int)
        self.name = Autoproperty(str)
        self.owner = Autoproperty(str)
        self.blurb = Autoproperty(str)
        self.file = Autoproperty(ModFileModel)


class InstalledModModel(BaseModModel):
    def __init__(self):
        super().__init__()
        self.mod_source = Autoproperty(str)
        self.dependency = Autoproperty(bool)


class ModModel(BaseModModel):
    def __init__(self):
        super().__init__()
        self.download_count = Autoproperty(int)
        self.categories = Autoproperty(list, [])


class ModFileModel(Automappable):
    def __init__(self):
        self.id = Autoproperty(int)
        self.download_url = Autoproperty(str)
        self.file_name = Autoproperty(str)
        self.file_md5 = Autoproperty(str)
        self.release_type = Autoproperty(str)
        self.mod_dependencies = Autoproperty(list, [])



import requests

from carrot_mc import meta
from carrot_mc.model import ModModel, ModFileModel

API_ENDPOINT = 'https://api.carrot-mc.xyz/prod/'


class BackendService:
    def search_by_mod_key(self, mod_key: str, mc_version: str) -> list:
        r = requests.get(
            url=f'{API_ENDPOINT}mods?mc_version={mc_version}&mod_key={mod_key}&page_size=30',
            headers=self._get_default_headers()
        )
        mods = [ModModel.from_dict(m) for m in r.json()['result']]
        return mods

    def get_mod_info(self, mod_key: str) -> ModModel:
        r = requests.get(
            url=f'{API_ENDPOINT}mod/{mod_key}',
            headers=self._get_default_headers()
        )
        mod = ModModel.from_dict(r.json()['result'])
        return mod

    def get_newest_file_info(self, mod_key: str, mc_version: str, channel: str) -> ModFileModel:
        r = requests.get(
            url=f'{API_ENDPOINT}mod/{mod_key}/files?mc_version={mc_version}&channel={channel}&newest_only=1',
            headers=self._get_default_headers()
        )
        result = r.json()['result']
        if result:
            file = ModFileModel.from_dict(result[0])
            return file

    def download_file(self, download_url: str) -> bytes:
        r = requests.get(
            url=download_url,
            headers=self._get_default_headers()
        )
        result = r.content
        return result

    def _get_default_headers(self):
        return {
            'User-Agent': 'Carrot ' + meta.VERSION
        }